@@ -14,7 +14,7 @@ import pymongo
 # Add any Sphinx extension module names here, as strings. They can be extensions
 # coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
 extensions = ['sphinx.ext.autodoc', 'sphinx.ext.doctest', 'sphinx.ext.coverage',
-              'sphinx.ext.todo']
+              'sphinx.ext.todo', 'doc.mongo_extensions']
 
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ['_templates']
