@@ -53,7 +53,7 @@ class GridFS(object):
         self.__collection = database[collection]
         self.__files = self.__collection.files
         self.__chunks = self.__collection.chunks
-        self.__chunks.create_index([("files_id", ASCENDING), ("n", ASCENDING)],
+        self.__chunks.ensure_index([("files_id", ASCENDING), ("n", ASCENDING)],
                                    unique=True)
 
     def new_file(self, **kwargs):
@@ -63,6 +63,10 @@ class GridFS(object):
         which data can be written. Any keyword arguments will be
         passed through to :meth:`~gridfs.grid_file.GridIn`.
 
+        If the ``"_id"`` of the file is manually specified, it must
+        not already exist in GridFS. Otherwise
+        :class:`~gridfs.errors.FileExists` is raised.
+
         :Parameters:
           - `**kwargs` (optional): keyword arguments for file creation
 
@@ -87,6 +91,10 @@ class GridFS(object):
         :meth:`~gridfs.grid_file.GridIn` for possible
         arguments. Returns the ``"_id"`` of the created file.
 
+        If the ``"_id"`` of the file is manually specified, it must
+        not already exist in GridFS. Otherwise
+        :class:`~gridfs.errors.FileExists` is raised.
+
         :Parameters:
           - `data`: data to be written as a file.
           - `**kwargs` (optional): keyword arguments for file creation
