@@ -23,10 +23,13 @@ except ImportError:
     from StringIO import StringIO
 
 from gridfs.errors import (CorruptGridFile,
+                           FileExists,
                            NoFile,
                            UnsupportedAPI)
+from pymongo import ASCENDING
 from pymongo.binary import Binary
 from pymongo.collection import Collection
+from pymongo.errors import DuplicateKeyError
 from pymongo.objectid import ObjectId
 
 try:
@@ -93,7 +96,8 @@ class GridIn(object):
         arguments include:
 
           - ``"_id"``: unique ID for this file (default:
-            :class:`~pymongo.objectid.ObjectId`)
+            :class:`~pymongo.objectid.ObjectId`) - this ``"_id"`` must
+            not have already been used for another file
 
           - ``"filename"``: human name for the file
 
@@ -103,10 +107,6 @@ class GridIn(object):
           - ``"chunkSize"`` or ``"chunk_size"``: size of each of the
             chunks, in bytes (default: 256 kb)
 
-          - ``"aliases"``: array of alias strings
-
-          - ``"metadata"``: document containing arbitrary metadata
-
         :Parameters:
           - `root_collection`: root collection to write to
           - `**kwargs` (optional): file level options (see above)
@@ -124,6 +124,8 @@ class GridIn(object):
         kwargs["_id"] = kwargs.get("_id", ObjectId())
         kwargs["chunkSize"] = kwargs.get("chunkSize", DEFAULT_CHUNK_SIZE)
 
+        root_collection.chunks.ensure_index([("files_id", ASCENDING), ("n", ASCENDING)],
+                                            unique=True)
         object.__setattr__(self, "_coll", root_collection)
         object.__setattr__(self, "_chunks", root_collection.chunks)
         object.__setattr__(self, "_file", kwargs)
@@ -140,7 +142,7 @@ class GridIn(object):
 
     _id = _create_property("_id", "The ``'_id'`` value for this file.",
                             read_only=True)
-    name = _create_property("filename", "Name of this file.")
+    filename = _create_property("filename", "Name of this file.")
     content_type = _create_property("contentType", "Mime-type for this file.")
     length = _create_property("length", "Length (in bytes) of this file.",
                                closed_only=True)
@@ -196,7 +198,12 @@ class GridIn(object):
         self._file["md5"] = md5
         self._file["length"] = self._position
         self._file["uploadDate"] = datetime.datetime.utcnow()
-        return self._coll.files.insert(self._file)
+
+        try:
+            return self._coll.files.insert(self._file, safe=True)
+        except DuplicateKeyError:
+            raise FileExists("file with _id %r already exists" % self._id)
+
 
     def close(self):
         """Flush the file and close it.
