@@ -25,12 +25,11 @@
 
 
 #include <stdio.h>
-#include <time.h>
 
 #include <Python.h>
 #include <datetime.h>
 
-#include "time_helpers.h"
+#include "time64.h"
 #include "encoding_helpers.h"
 
 static PyObject* SON = NULL;
@@ -41,6 +40,9 @@ static PyObject* DBRef = NULL;
 static PyObject* RECompile = NULL;
 static PyObject* UUID = NULL;
 static PyObject* Timestamp = NULL;
+static PyObject* MinKey = NULL;
+static PyObject* MaxKey = NULL;
+static PyTypeObject* REType = NULL;
 
 #if PY_VERSION_HEX < 0x02050000 && !defined(PY_SSIZE_T_MIN)
 typedef int Py_ssize_t;
@@ -48,6 +50,14 @@ typedef int Py_ssize_t;
 #define PY_SSIZE_T_MIN INT_MIN
 #endif
 
+#if PY_VERSION_HEX < 0x02050000
+#define WARN(category, message)                 \
+    PyErr_Warn((category), (message))
+#else
+#define WARN(category, message)                 \
+    PyErr_WarnEx((category), (message), 1)
+#endif
+
 #define INITIAL_BUFFER_SIZE 256
 
 /* Maximum number of regex flags */
@@ -87,6 +97,40 @@ typedef int Py_ssize_t;
 #define STRCAT(dest, n, src) strcat((dest), (src))
 #endif
 
+
+/* Date stuff */
+static PyObject* datetime_from_millis(long long millis) {
+    int microseconds = (millis % 1000) * 1000;
+    Time64_T seconds = millis / 1000;
+    struct TM timeinfo;
+    gmtime64_r(&seconds, &timeinfo);
+
+    return PyDateTime_FromDateAndTime(timeinfo.tm_year + 1900,
+                                      timeinfo.tm_mon + 1,
+                                      timeinfo.tm_mday,
+                                      timeinfo.tm_hour,
+                                      timeinfo.tm_min,
+                                      timeinfo.tm_sec,
+                                      microseconds);
+}
+
+static long long millis_from_datetime(PyObject* datetime) {
+    struct TM timeinfo;
+    long long millis;
+
+    timeinfo.tm_year = PyDateTime_GET_YEAR(datetime) - 1900;
+    timeinfo.tm_mon = PyDateTime_GET_MONTH(datetime) - 1;
+    timeinfo.tm_mday = PyDateTime_GET_DAY(datetime);
+    timeinfo.tm_hour = PyDateTime_DATE_GET_HOUR(datetime);
+    timeinfo.tm_min = PyDateTime_DATE_GET_MINUTE(datetime);
+    timeinfo.tm_sec = PyDateTime_DATE_GET_SECOND(datetime);
+
+    millis = timegm64(&timeinfo) * 1000;
+    millis += PyDateTime_DATE_GET_MICROSECOND(datetime) / 1000;
+    return millis;
+}
+
+
 /* A buffer representing some data being encoded to BSON. */
 typedef struct {
     char* buffer;
@@ -96,7 +140,7 @@ typedef struct {
 
 static int write_dict(bson_buffer* buffer, PyObject* dict,
                       unsigned char check_keys, unsigned char top_level);
-static PyObject* elements_to_dict(const char* string, int max);
+static PyObject* elements_to_dict(const char* string, int max, PyObject* as_class);
 
 static bson_buffer* buffer_new(void) {
     bson_buffer* buffer;
@@ -230,6 +274,8 @@ static int _reload_python_objects(void) {
         _reload_object(&ObjectId, "pymongo.objectid", "ObjectId") ||
         _reload_object(&DBRef, "pymongo.dbref", "DBRef") ||
         _reload_object(&Timestamp, "pymongo.timestamp", "Timestamp") ||
+        _reload_object(&MinKey, "pymongo.min_key", "MinKey") ||
+        _reload_object(&MaxKey, "pymongo.max_key", "MaxKey") ||
         _reload_object(&RECompile, "re", "compile")) {
         return 1;
     }
@@ -238,6 +284,9 @@ static int _reload_python_objects(void) {
         UUID = NULL;
         PyErr_Clear();
     }
+    /* Reload our REType hack too. */
+    REType = PyObject_CallFunction(RECompile, "O",
+                                   PyString_FromString(""))->ob_type;
     return 0;
 }
 
@@ -463,26 +512,21 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         Py_DECREF(encoded);
         return result;
     } else if (PyDateTime_CheckExact(value)) {
-        time_t rawtime;
-        struct tm timeinfo;
-        long long time_since_epoch;
-
-        time(&rawtime);
-        if (LOCALTIME(&timeinfo, &rawtime)) {
-            return 0;
+        long long millis;
+        PyObject* utcoffset = PyObject_CallMethod(value, "utcoffset", NULL);
+        if (utcoffset != Py_None) {
+            PyObject* result = PyNumber_Subtract(value, utcoffset);
+            Py_DECREF(utcoffset);
+            if (!result) {
+                return 0;
+            }
+            millis = millis_from_datetime(result);
+            Py_DECREF(result);
+        } else {
+            millis = millis_from_datetime(value);
         }
-        timeinfo.tm_year = PyDateTime_GET_YEAR(value) - 1900;
-        timeinfo.tm_mon = PyDateTime_GET_MONTH(value) - 1;
-        timeinfo.tm_mday = PyDateTime_GET_DAY(value);
-        timeinfo.tm_hour = PyDateTime_DATE_GET_HOUR(value);
-        timeinfo.tm_min = PyDateTime_DATE_GET_MINUTE(value);
-        timeinfo.tm_sec = PyDateTime_DATE_GET_SECOND(value);
-        time_since_epoch = GMTIME_INVERSE(&timeinfo);
-        time_since_epoch = time_since_epoch * 1000;
-        time_since_epoch += PyDateTime_DATE_GET_MICROSECOND(value) / 1000;
-
         *(buffer->buffer + type_byte) = 0x09;
-        return buffer_write_bytes(buffer, (const char*)&time_since_epoch, 8);
+        return buffer_write_bytes(buffer, (const char*)&millis, 8);
     } else if (PyObject_IsInstance(value, ObjectId)) {
         PyObject* pystring = PyObject_GetAttrString(value, "_ObjectId__id");
         if (!pystring) {
@@ -541,8 +585,7 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         *(buffer->buffer + type_byte) = 0x11;
         return 1;
     }
-    else if (PyObject_HasAttrString(value, "pattern") &&
-             PyObject_HasAttrString(value, "flags")) { /* TODO just a proxy for checking if it is a compiled re */
+    else if (PyObject_TypeCheck(value, REType)) {
         PyObject* py_flags = PyObject_GetAttrString(value, "flags");
         PyObject* py_pattern;
         PyObject* encoded_pattern;
@@ -627,14 +670,19 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         }
         *(buffer->buffer + type_byte) = 0x0B;
         return 1;
+    } else if (PyObject_IsInstance(value, MinKey)) {
+        *(buffer->buffer + type_byte) = 0xFF;
+        return 1;
+    } else if (PyObject_IsInstance(value, MaxKey)) {
+        *(buffer->buffer + type_byte) = 0x7F;
+        return 1;
     } else if (first_attempt) {
         /* Try reloading the modules and having one more go at it. */
-        if (PyErr_WarnEx(PyExc_RuntimeWarning, "couldn't encode - reloading "
-                         "python modules and trying again. if you see this "
-                         "without getting an InvalidDocument exception "
-                         "please see "
-                         "http://api.mongodb.org/python/current/faq.html#does-pymongo-work-with-mod-wsgi",
-                         1) == -1) {
+        if (WARN(PyExc_RuntimeWarning, "couldn't encode - reloading python "
+                 "modules and trying again. if you see this without getting "
+                 "an InvalidDocument exception please see http://api.mongodb"
+                 ".org/python/current/faq.html#does-pymongo-work-with-mod-"
+                 "wsgi") == -1) {
             return 0;
         }
         if (_reload_python_objects()) {
@@ -1191,7 +1239,8 @@ static PyObject* _cbson_get_more_message(PyObject* self, PyObject* args) {
     return result;
 }
 
-static PyObject* get_value(const char* buffer, int* position, int type) {
+static PyObject* get_value(const char* buffer, int* position, int type,
+                           PyObject* as_class) {
     PyObject* value;
     switch (type) {
     case 1:
@@ -1222,7 +1271,7 @@ static PyObject* get_value(const char* buffer, int* position, int type) {
         {
             int size;
             memcpy(&size, buffer + *position, 4);
-            value = elements_to_dict(buffer + *position + 4, size - 5);
+            value = elements_to_dict(buffer + *position + 4, size - 5, as_class);
             if (!value) {
                 return NULL;
             }
@@ -1260,7 +1309,7 @@ static PyObject* get_value(const char* buffer, int* position, int type) {
                 int type = (int)buffer[(*position)++];
                 int key_size = strlen(buffer + *position);
                 *position += key_size + 1; /* just skip the key, they're in order. */
-                to_append = get_value(buffer, position, type);
+                to_append = get_value(buffer, position, type, as_class);
                 if (!to_append) {
                     return NULL;
                 }
@@ -1355,25 +1404,7 @@ static PyObject* get_value(const char* buffer, int* position, int type) {
         }
     case 9:
         {
-            long long millis;
-            int microseconds;
-            time_t seconds;
-            struct tm timeinfo;
-
-            memcpy(&millis, buffer + *position, 8);
-            microseconds = (millis % 1000) * 1000;
-            seconds = millis / 1000;
-            if (GMTIME(&timeinfo, &seconds)) {
-                return NULL;
-            }
-
-            value = PyDateTime_FromDateAndTime(timeinfo.tm_year + 1900,
-                                               timeinfo.tm_mon + 1,
-                                               timeinfo.tm_mday,
-                                               timeinfo.tm_hour,
-                                               timeinfo.tm_min,
-                                               timeinfo.tm_sec,
-                                               microseconds);
+            value = datetime_from_millis(*(long long*)(buffer + *position));
             *position += 8;
             break;
         }
@@ -1451,7 +1482,7 @@ static PyObject* get_value(const char* buffer, int* position, int type) {
             *position += code_length + 1;
 
             memcpy(&scope_size, buffer + *position, 4);
-            scope = elements_to_dict(buffer + *position + 4, scope_size - 5);
+            scope = elements_to_dict(buffer + *position + 4, scope_size - 5, (PyObject*)&PyDict_Type);
             if (!scope) {
                 Py_DECREF(code);
                 return NULL;
@@ -1476,11 +1507,10 @@ static PyObject* get_value(const char* buffer, int* position, int type) {
         }
     case 17:
         {
-            int time,
-                inc;
+            unsigned int time, inc;
             memcpy(&inc, buffer + *position, 4);
             memcpy(&time, buffer + *position + 4, 4);
-            value = PyObject_CallFunction(Timestamp, "ii", time, inc);
+            value = PyObject_CallFunction(Timestamp, "II", time, inc);
             if (!value) {
                 return NULL;
             }
@@ -1498,6 +1528,16 @@ static PyObject* get_value(const char* buffer, int* position, int type) {
             *position += 8;
             break;
         }
+    case -1:
+        {
+            value = PyObject_CallFunctionObjArgs(MinKey, NULL);
+            break;
+        }
+    case 127:
+        {
+            value = PyObject_CallFunctionObjArgs(MaxKey, NULL);
+            break;
+        }
     default:
         {
             PyObject* InvalidDocument = _error("InvalidDocument");
@@ -1509,9 +1549,9 @@ static PyObject* get_value(const char* buffer, int* position, int type) {
     return value;
 }
 
-static PyObject* elements_to_dict(const char* string, int max) {
+static PyObject* elements_to_dict(const char* string, int max, PyObject* as_class) {
     int position = 0;
-    PyObject* dict = PyDict_New();
+    PyObject* dict = PyObject_CallObject(as_class, NULL);
     if (!dict) {
         return NULL;
     }
@@ -1524,38 +1564,68 @@ static PyObject* elements_to_dict(const char* string, int max) {
             return NULL;
         }
         position += name_length + 1;
-        value = get_value(string, &position, type);
+        value = get_value(string, &position, type, as_class);
         if (!value) {
             return NULL;
         }
 
-        PyDict_SetItem(dict, name, value);
+        PyObject_SetItem(dict, name, value);
         Py_DECREF(name);
         Py_DECREF(value);
     }
     return dict;
 }
 
-static PyObject* _cbson_bson_to_dict(PyObject* self, PyObject* bson) {
-    int size;
+static PyObject* _cbson_bson_to_dict(PyObject* self, PyObject* args) {
+    unsigned int size;
     Py_ssize_t total_size;
     const char* string;
+    PyObject* bson;
+    PyObject* as_class;
     PyObject* dict;
     PyObject* remainder;
     PyObject* result;
 
+    if (!PyArg_ParseTuple(args, "OO", &bson, &as_class)) {
+        return NULL;
+    }
+
     if (!PyString_Check(bson)) {
         PyErr_SetString(PyExc_TypeError, "argument to _bson_to_dict must be a string");
         return NULL;
     }
     total_size = PyString_Size(bson);
+    if (total_size < 5) {
+        PyObject* InvalidBSON = _error("InvalidBSON");
+        PyErr_SetString(InvalidBSON,
+                        "not enough data for a BSON document");
+        Py_DECREF(InvalidBSON);
+        return NULL;
+    }
+
     string = PyString_AsString(bson);
     if (!string) {
         return NULL;
     }
     memcpy(&size, string, 4);
 
-    dict = elements_to_dict(string + 4, size - 5);
+    if (total_size < size) {
+        PyObject* InvalidBSON = _error("InvalidBSON");
+        PyErr_SetString(InvalidBSON,
+                        "objsize too large");
+        Py_DECREF(InvalidBSON);
+        return NULL;
+    }
+
+    if (string[size - 1]) {
+        PyObject* InvalidBSON = _error("InvalidBSON");
+        PyErr_SetString(InvalidBSON,
+                        "bad eoo");
+        Py_DECREF(InvalidBSON);
+        return NULL;
+    }
+
+    dict = elements_to_dict(string + 4, size - 5, as_class);
     if (!dict) {
         return NULL;
     }
@@ -1570,12 +1640,18 @@ static PyObject* _cbson_bson_to_dict(PyObject* self, PyObject* bson) {
     return result;
 }
 
-static PyObject* _cbson_to_dicts(PyObject* self, PyObject* bson) {
-    int size;
+static PyObject* _cbson_to_dicts(PyObject* self, PyObject* args) {
+    unsigned int size;
     Py_ssize_t total_size;
     const char* string;
+    PyObject* bson;
     PyObject* dict;
     PyObject* result;
+    PyObject* as_class = (PyObject*)&PyDict_Type;
+
+    if (!PyArg_ParseTuple(args, "O|O", &bson, &as_class)) {
+        return NULL;
+    }
 
     if (!PyString_Check(bson)) {
         PyErr_SetString(PyExc_TypeError, "argument to _to_dicts must be a string");
@@ -1590,9 +1666,33 @@ static PyObject* _cbson_to_dicts(PyObject* self, PyObject* bson) {
     result = PyList_New(0);
 
     while (total_size > 0) {
+        if (total_size < 5) {
+            PyObject* InvalidBSON = _error("InvalidBSON");
+            PyErr_SetString(InvalidBSON,
+                            "not enough data for a BSON document");
+            Py_DECREF(InvalidBSON);
+            return NULL;
+        }
+
         memcpy(&size, string, 4);
 
-        dict = elements_to_dict(string + 4, size - 5);
+        if (total_size < size) {
+            PyObject* InvalidBSON = _error("InvalidBSON");
+            PyErr_SetString(InvalidBSON,
+                            "objsize too large");
+            Py_DECREF(InvalidBSON);
+            return NULL;
+        }
+
+        if (string[size - 1]) {
+            PyObject* InvalidBSON = _error("InvalidBSON");
+            PyErr_SetString(InvalidBSON,
+                            "bad eoo");
+            Py_DECREF(InvalidBSON);
+            return NULL;
+        }
+
+        dict = elements_to_dict(string + 4, size - 5, as_class);
         if (!dict) {
             return NULL;
         }
@@ -1608,9 +1708,9 @@ static PyObject* _cbson_to_dicts(PyObject* self, PyObject* bson) {
 static PyMethodDef _CBSONMethods[] = {
     {"_dict_to_bson", _cbson_dict_to_bson, METH_VARARGS,
      "convert a dictionary to a string containing it's BSON representation."},
-    {"_bson_to_dict", _cbson_bson_to_dict, METH_O,
+    {"_bson_to_dict", _cbson_bson_to_dict, METH_VARARGS,
      "convert a BSON string to a SON object."},
-    {"_to_dicts", _cbson_to_dicts, METH_O,
+    {"_to_dicts", _cbson_to_dicts, METH_VARARGS,
      "convert binary data to a sequence of SON objects."},
     {"_insert_message", _cbson_insert_message, METH_VARARGS,
      "create an insert message to be sent to MongoDB"},
