@@ -15,14 +15,12 @@
 """Collection level utilities for Mongo."""
 
 import warnings
-import struct
 
 from pymongo import (helpers,
                      message)
 from pymongo.code import Code
 from pymongo.cursor import Cursor
 from pymongo.errors import InvalidName
-from pymongo.objectid import ObjectId
 from pymongo.son import SON
 
 _ZERO = "\x00\x00\x00\x00"
@@ -99,13 +97,6 @@ class Collection(object):
         self.__database = database
         self.__name = unicode(name)
         self.__full_name = u"%s.%s" % (self.__database.name, self.__name)
-        # TODO remove the callable_value wrappers after deprecation is complete
-        self.__database_w = helpers.callable_value(self.__database,
-                                                   "Collection.database")
-        self.__name_w = helpers.callable_value(self.__name,
-                                               "Collection.name")
-        self.__full_name_w = helpers.callable_value(self.__full_name,
-                                                    "Collection.full_name")
         if create or options is not None:
             self.__create(options)
 
@@ -151,20 +142,18 @@ class Collection(object):
         The full name is of the form `database_name.collection_name`.
 
         .. versionchanged:: 1.3
-           ``full_name`` is now a property rather than a method. The
-           ``full_name()`` method is deprecated.
+           ``full_name`` is now a property rather than a method.
         """
-        return self.__full_name_w
+        return self.__full_name
 
     @property
     def name(self):
         """The name of this :class:`Collection`.
 
         .. versionchanged:: 1.3
-           ``name`` is now a property rather than a method. The
-           ``name()`` method is deprecated.
+           ``name`` is now a property rather than a method.
         """
-        return self.__name_w
+        return self.__name
 
     @property
     def database(self):
@@ -172,10 +161,9 @@ class Collection(object):
         :class:`Collection` is a part of.
 
         .. versionchanged:: 1.3
-           ``database`` is now a property rather than a method. The
-           ``database()`` method is deprecated.
+           ``database`` is now a property rather than a method.
         """
-        return self.__database_w
+        return self.__database
 
     def save(self, to_save, manipulate=True, safe=False):
         """Save a document in this collection.
@@ -324,40 +312,40 @@ class Collection(object):
             message.update(self.__full_name, upsert, multi,
                            spec, document, safe), safe)
 
-    def remove(self, spec_or_object_id=None, safe=False):
+    def remove(self, spec_or_id=None, safe=False):
         """Remove a document(s) from this collection.
 
         .. warning:: Calls to :meth:`remove` should be performed with
            care, as removed data cannot be restored.
 
-        Raises :class:`~pymongo.errors.TypeError` if
-        `spec_or_object_id` is not an instance of (``dict``,
-        :class:`~pymongo.objectid.ObjectId`). If `safe` is ``True``
-        then the remove operation will be checked for errors, raising
+        If `safe` is ``True`` then the remove operation will be
+        checked for errors, raising
         :class:`~pymongo.errors.OperationFailure` if one
         occurred. Safe removes wait for a response from the database,
         while normal removes do not.
 
-        If no `spec_or_object_id` is given all documents in this
-        collection will be removed. This is not equivalent to calling
-        :meth:`~pymongo.database.Database.drop_collection`, however, as
-        indexes will not be removed.
+        If `spec_or_id` is ``None``, all documents in this collection
+        will be removed. This is not equivalent to calling
+        :meth:`~pymongo.database.Database.drop_collection`, however,
+        as indexes will not be removed.
 
         If `safe` is ``True`` returns the response to the *lastError*
         command. Otherwise, returns ``None``.
 
         :Parameters:
-          - `spec_or_object_id` (optional): a ``dict`` or
-            :class:`~pymongo.son.SON` instance specifying which documents
-            should be removed; or an instance of
-            :class:`~pymongo.objectid.ObjectId` specifying the value of the
-            ``_id`` field for the document to be removed
+          - `spec_or_id` (optional): a dictionary specifying the
+            documents to be removed OR any other type specifying the
+            value of ``"_id"`` for the document to be removed
           - `safe` (optional): check that the remove succeeded?
 
+        .. versionchanged:: 1.7
+           Accept any type other than a ``dict`` instance for removal
+           by ``"_id"``, not just :class:`~pymongo.objectid.ObjectId`
+           instances.
         .. versionchanged:: 1.4
            Return the response to *lastError* if `safe` is ``True``.
         .. versionchanged:: 1.2
-           The `spec_or_object_id` parameter is now optional. If it is
+           The `spec_or_id` parameter is now optional. If it is
            not specified *all* documents in the collection will be
            removed.
         .. versionadded:: 1.1
@@ -365,141 +353,125 @@ class Collection(object):
 
         .. mongodoc:: remove
         """
-        spec = spec_or_object_id
-        if spec is None:
-            spec = {}
-        if isinstance(spec, ObjectId):
-            spec = {"_id": spec}
-
-        if not isinstance(spec, dict):
-            raise TypeError("spec must be an instance of dict, not %s" %
-                            type(spec))
+        if spec_or_id is None:
+            spec_or_id = {}
+        if not isinstance(spec_or_id, dict):
+            spec_or_id = {"_id": spec_or_id}
 
         return self.__database.connection._send_message(
-            message.delete(self.__full_name, spec, safe), safe)
+            message.delete(self.__full_name, spec_or_id, safe), safe)
 
-    def find_one(self, spec_or_object_id=None, fields=None,
-                 _sock=None, _must_use_master=False, _is_command=False):
-        """Get a single object from the database.
+    def find_one(self, spec_or_id=None, *args, **kwargs):
+        """Get a single document from the database.
 
-        Raises TypeError if the argument is of an improper type. Returns a
-        single SON object, or None if no result is found.
+        All arguments to :meth:`find` are also valid arguments for
+        :meth:`find_one`, although any `limit` argument will be
+        ignored. Returns a single document, or ``None`` if no matching
+        document is found.
 
         :Parameters:
-          - `spec_or_object_id` (optional): a SON object specifying elements
-            which must be present for a document to be returned OR an instance
-            of ObjectId to be used as the value for an _id query
-          - `fields` (optional): a list of field names that should be included
-            in the returned document ("_id" will always be included)
-        """
-        spec = spec_or_object_id
-        if spec is None:
-            spec = SON()
-        if isinstance(spec, ObjectId):
-            spec = SON({"_id": spec})
 
-        for result in self.find(spec, limit=-1, fields=fields,
-                                _sock=_sock, _must_use_master=_must_use_master,
-                                _is_command=_is_command):
+          - `spec_or_id` (optional): a dictionary specifying
+            the query to be performed OR any other type to be used as
+            the value for a query for ``"_id"``.
+
+          - `*args` (optional): any additional positional arguments
+            are the same as the arguments to :meth:`find`.
+
+          - `**kwargs` (optional): any additional keyword arguments
+            are the same as the arguments to :meth:`find`.
+
+        .. versionchanged:: 1.7
+           Allow passing any of the arguments that are valid for
+           :meth:`find`.
+
+        .. versionchanged:: 1.7
+           Accept any type other than a ``dict`` instance as an
+           ``"_id"`` query, not just
+           :class:`~pymongo.objectid.ObjectId` instances.
+        """
+        if spec_or_id is not None and not isinstance(spec_or_id, dict):
+            spec_or_id = {"_id": spec_or_id}
+
+        for result in self.find(spec_or_id, *args, **kwargs).limit(-1):
             return result
         return None
 
-    def _fields_list_to_dict(self, fields):
-        """Takes a list of field names and returns a matching dictionary.
-
-        ["a", "b"] becomes {"a": 1, "b": 1}
-
-        and
-
-        ["a.b.c", "d", "a.c"] becomes {"a.b.c": 1, "d": 1, "a.c": 1}
-        """
-        as_dict = {}
-        for field in fields:
-            if not isinstance(field, basestring):
-                raise TypeError("fields must be a list of key names as "
-                                "(string, unicode)")
-            as_dict[field] = 1
-        return as_dict
-
-    def find(self, spec=None, fields=None, skip=0, limit=0,
-             timeout=True, snapshot=False, tailable=False,
-             _sock=None, _must_use_master=False, _is_command=False):
+    def find(self, *args, **kwargs):
         """Query the database.
 
-        The `spec` argument is a prototype document that all results must
-        match. For example:
+        The `spec` argument is a prototype document that all results
+        must match. For example:
 
         >>> db.test.find({"hello": "world"})
 
-        only matches documents that have a key "hello" with value "world".
-        Matches can have other keys *in addition* to "hello". The `fields`
-        argument is used to specify a subset of fields that should be included
-        in the result documents. By limiting results to a certain subset of
-        fields you can cut down on network traffic and decoding time.
+        only matches documents that have a key "hello" with value
+        "world".  Matches can have other keys *in addition* to
+        "hello". The `fields` argument is used to specify a subset of
+        fields that should be included in the result documents. By
+        limiting results to a certain subset of fields you can cut
+        down on network traffic and decoding time.
 
-        Raises TypeError if any of the arguments are of improper type. Returns
-        an instance of Cursor corresponding to this query.
+        Raises :class:`TypeError` if any of the arguments are of
+        improper type. Returns an instance of
+        :class:`~pymongo.cursor.Cursor` corresponding to this query.
 
         :Parameters:
-          - `spec` (optional): a SON object specifying elements which must be
-            present for a document to be included in the result set
-          - `fields` (optional): a list of field names that should be returned
-            in the result set ("_id" will always be included)
-          - `skip` (optional): the number of documents to omit (from the start
-            of the result set) when returning the results
-          - `limit` (optional): the maximum number of results to return
-          - `timeout` (optional): if True, any returned cursor will be subject
-            to the normal timeout behavior of the mongod process. Otherwise,
-            the returned cursor will never timeout at the server. Care should
-            be taken to ensure that cursors with timeout turned off are
-            properly closed.
-          - `snapshot` (optional): if True, snapshot mode will be used for this
-            query. Snapshot mode assures no duplicates are returned, or objects
-            missed, which were present at both the start and end of the query's
-            execution. For details, see the `snapshot documentation
+          - `spec` (optional): a SON object specifying elements which
+            must be present for a document to be included in the
+            result set
+          - `fields` (optional): a list of field names that should be
+            returned in the result set ("_id" will always be
+            included), or a dict specifying the `fields to return
+            <http://www.mongodb.org/display/DOCS/Advanced+Queries#AdvancedQueries-RetrievingaSubsetofFields>`_
+          - `skip` (optional): the number of documents to omit (from
+            the start of the result set) when returning the results
+          - `limit` (optional): the maximum number of results to
+            return
+          - `timeout` (optional): if True, any returned cursor will be
+            subject to the normal timeout behavior of the mongod
+            process. Otherwise, the returned cursor will never timeout
+            at the server. Care should be taken to ensure that cursors
+            with timeout turned off are properly closed.
+          - `snapshot` (optional): if True, snapshot mode will be used
+            for this query. Snapshot mode assures no duplicates are
+            returned, or objects missed, which were present at both
+            the start and end of the query's execution. For details,
+            see the `snapshot documentation
             <http://www.mongodb.org/display/DOCS/How+to+do+Snapshotting+in+the+Mongo+Database>`_.
-          - `tailable` (optional): the result of this find call will be a
-            tailable cursor - tailable cursors aren't closed when the last data
-            is retrieved but are kept open and the cursors location marks the
-            final document's position. if more data is received iteration of
-            the cursor will continue from the last document received. For
-            details, see the `tailable cursor documentation
+          - `tailable` (optional): the result of this find call will
+            be a tailable cursor - tailable cursors aren't closed when
+            the last data is retrieved but are kept open and the
+            cursors location marks the final document's position. if
+            more data is received iteration of the cursor will
+            continue from the last document received. For details, see
+            the `tailable cursor documentation
             <http://www.mongodb.org/display/DOCS/Tailable+Cursors>`_.
+          - `sort` (optional): a list of (key, direction) pairs
+            specifying the sort order for this query. See
+            :meth:`~pymongo.cursor.Cursor.sort` for details.
+          - `max_scan` (optional): limit the number of documents
+            examined when performing the query
+          - `as_class` (optional): class to use for documents in the
+            query result (default is
+            :attr:`~pymongo.connection.Connection.document_class`)
+
+        .. note:: The `max_scan` parameter requires server
+           version **>= 1.5.1**
+
+        .. versionadded:: 1.7
+           The `sort`, `max_scan` and `as_class` parameters.
+
+        .. versionchanged:: 1.7
+           The `fields` parameter can now be a dict or any iterable in
+           addition to a list.
 
         .. versionadded:: 1.1
            The `tailable` parameter.
 
         .. mongodoc:: find
         """
-        if spec is None:
-            spec = SON()
-
-        slave_okay = self.__database.connection.slave_okay
-
-        if not isinstance(spec, dict):
-            raise TypeError("spec must be an instance of dict")
-        if fields is not None and not isinstance(fields, list):
-            raise TypeError("fields must be an instance of list")
-        if not isinstance(skip, int):
-            raise TypeError("skip must be an instance of int")
-        if not isinstance(limit, int):
-            raise TypeError("limit must be an instance of int")
-        if not isinstance(timeout, bool):
-            raise TypeError("timeout must be an instance of bool")
-        if not isinstance(snapshot, bool):
-            raise TypeError("snapshot must be an instance of bool")
-        if not isinstance(tailable, bool):
-            raise TypeError("tailable must be an instance of bool")
-
-        if fields is not None:
-            if not fields:
-                fields = ["_id"]
-            fields = self._fields_list_to_dict(fields)
-
-        return Cursor(self, spec, fields, skip, limit, slave_okay, timeout,
-                      tailable, snapshot, _sock=_sock,
-                      _must_use_master=_must_use_master,
-                      _is_command=_is_command)
+        return Cursor(self, *args, **kwargs)
 
     def count(self):
         """Get the number of documents in this collection.
@@ -509,7 +481,8 @@ class Collection(object):
         """
         return self.find().count()
 
-    def create_index(self, key_or_list, deprecated_unique=None, ttl=300, **kwargs):
+    def create_index(self, key_or_list, deprecated_unique=None,
+                     ttl=300, **kwargs):
         """Creates an index on this collection.
 
         Takes either a single key or a list of (key, direction) pairs.
@@ -550,8 +523,9 @@ class Collection(object):
             this index will be recognized by subsequent calls to
             :meth:`ensure_index` - see documentation for
             :meth:`ensure_index` for details
-          - `kwargs` (optional): any additional index creation options
-            (see the above list) should be passed as keyword arguments
+          - `**kwargs` (optional): any additional index creation
+            options (see the above list) should be passed as keyword
+            arguments
 
         .. versionchanged:: 1.5.1
            Accept kwargs to support all index creation options.
@@ -591,7 +565,8 @@ class Collection(object):
                                               check_keys=False)
         return name
 
-    def ensure_index(self, key_or_list, deprecated_unique=None, ttl=300, **kwargs):
+    def ensure_index(self, key_or_list, deprecated_unique=None,
+                     ttl=300, **kwargs):
         """Ensures that an index exists on this collection.
 
         Takes either a single key or a list of (key, direction) pairs.
@@ -638,8 +613,9 @@ class Collection(object):
           - `ttl` (optional): time window (in seconds) during which
             this index will be recognized by subsequent calls to
             :meth:`ensure_index`
-          - `kwargs` (optional): any additional index creation options
-            (see the above list) should be passed as keyword arguments
+          - `**kwargs` (optional): any additional index creation
+            options (see the above list) should be passed as keyword
+            arguments
 
         .. versionchanged:: 1.5.1
            Accept kwargs to support all index creation options.
@@ -657,7 +633,8 @@ class Collection(object):
 
         if self.__database.connection._cache_index(self.__database.name,
                                                    self.__name, name, ttl):
-            return self.create_index(key_or_list, deprecated_unique, ttl, **kwargs)
+            return self.create_index(key_or_list, deprecated_unique,
+                                     ttl, **kwargs)
         return None
 
     def drop_indexes(self):
@@ -696,20 +673,41 @@ class Collection(object):
 
         self.__database.connection._purge_index(self.__database.name,
                                                 self.__name, name)
-        self.__database.command("deleteIndexes", self.__name, index=name,
+        self.__database.command("dropIndexes", self.__name, index=name,
                                 allowable_errors=["ns not found"])
 
     def index_information(self):
         """Get information on this collection's indexes.
 
-        Returns a dictionary where the keys are index names (as returned by
-        create_index()) and the values are lists of (key, direction) pairs
-        specifying the index (as passed to create_index()).
+        Returns a dictionary where the keys are index names (as
+        returned by create_index()) and the values are dictionaries
+        containing information about each index. The dictionary is
+        guaranteed to contain at least a single key, ``"key"`` which
+        is a list of (key, direction) pairs specifying the index (as
+        passed to create_index()). It will also contain any other
+        information in `system.indexes`, except for the ``"ns"`` and
+        ``"name"`` keys, which are cleaned. Example output might look
+        like this:
+
+        >>> db.test.ensure_index("x", unique=True)
+        u'x_1'
+        >>> db.test.index_information()
+        {u'_id_': {u'key': [(u'_id', 1)]},
+         u'x_1': {u'unique': True, u'key': [(u'x', 1)]}}
+
+
+        .. versionchanged:: 1.7
+           The values in the resultant dictionary are now dictionaries
+           themselves, whose ``"key"`` item contains the list that was
+           the value in previous versions of PyMongo.
         """
-        raw = self.__database.system.indexes.find({"ns": self.__full_name})
+        raw = self.__database.system.indexes.find({"ns": self.__full_name},
+                                                  {"ns": 0}, as_class=SON)
         info = {}
         for index in raw:
-            info[index["name"]] = index["key"].items()
+            index["key"] = index["key"].items()
+            index = dict(index)
+            info[index.pop("name")] = index
         return info
 
     def options(self):
@@ -774,7 +772,7 @@ class Collection(object):
         if isinstance(key, basestring):
             group["$keyf"] = Code(key)
         elif key is not None:
-            group = {"key": self._fields_list_to_dict(key)}
+            group = {"key": helpers._fields_list_to_dict(key)}
         group["ns"] = self.__name
         group["$reduce"] = Code(reduce)
         group["cond"] = condition
@@ -784,7 +782,7 @@ class Collection(object):
 
         return self.__database.command("group", group)["retval"]
 
-    def rename(self, new_name):
+    def rename(self, new_name, **kwargs):
         """Rename this collection.
 
         If operating in auth mode, client must be authorized as an
@@ -795,20 +793,27 @@ class Collection(object):
 
         :Parameters:
           - `new_name`: new name for this collection
+          - `**kwargs` (optional): any additional rename options
+            should be passed as keyword arguments
+            (i.e. ``dropTarget=True``)
+
+        .. versionadded:: 1.7
+           support for accepting keyword arguments for rename options
         """
         if not isinstance(new_name, basestring):
             raise TypeError("new_name must be an instance of basestring")
 
         if not new_name or ".." in new_name:
             raise InvalidName("collection names cannot be empty")
-        if "$" in new_name:
-            raise InvalidName("collection names must not contain '$'")
         if new_name[0] == "." or new_name[-1] == ".":
             raise InvalidName("collecion names must not start or end with '.'")
+        if "$" in new_name and not new_name.startswith("oplog.$main"):
+            raise InvalidName("collection names must not contain '$'")
 
         new_name = "%s.%s" % (self.__database.name, new_name)
         self.__database.connection.admin.command("renameCollection",
-                                                 self.__full_name, to=new_name)
+                                                 self.__full_name,
+                                                 to=new_name, **kwargs)
 
     def distinct(self, key):
         """Get a list of distinct values for `key` among all documents
