@@ -1,27 +1,28 @@
 # Copyright 2009-2010 10gen, Inc.
 #
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# Licensed under the Apache License, Version 2.0 (the "License"); you
+# may not use this file except in compliance with the License.  You
+# may obtain a copy of the License at
 #
 # http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
+# implied.  See the License for the specific language governing
+# permissions and limitations under the License.
 
 """Tools for connecting to MongoDB.
 
-To connect to a single instance of MongoDB use :class:`Connection`. To connect
-to a replica pair use :meth:`~Connection.paired`.
+To connect to a single instance of MongoDB use :class:`Connection`. To
+connect to a replica pair use :meth:`~Connection.paired`.
 
-.. seealso:: Module :mod:`~pymongo.master_slave_connection` for connecting to
-   master-slave clusters.
+.. seealso:: Module :mod:`~pymongo.master_slave_connection` for
+   connecting to master-slave clusters.
 
 To get a :class:`~pymongo.database.Database` instance from a
-:class:`Connection` use either dictionary-style or attribute-style access:
+:class:`Connection` use either dictionary-style or attribute-style
+access:
 
 .. doctest::
 
@@ -34,17 +35,13 @@ To get a :class:`~pymongo.database.Database` instance from a
 """
 
 import datetime
-import errno
 import os
-import random
 import socket
 import struct
-import sys
 import threading
 import warnings
 
-from pymongo import (bson,
-                     database,
+from pymongo import (database,
                      helpers,
                      message)
 from pymongo.cursor_manager import CursorManager
@@ -54,7 +51,6 @@ from pymongo.errors import (AutoReconnect,
                             DuplicateKeyError,
                             InvalidURI,
                             OperationFailure)
-from pymongo.son import SON
 
 _CONNECT_TIMEOUT = 20.0
 
@@ -98,7 +94,7 @@ class Pool(threading.local):
         self.sock = None
 
 
-class Connection(object): # TODO support auth for pooling
+class Connection(object):  # TODO support auth for pooling
     """Connection to MongoDB.
     """
 
@@ -107,7 +103,7 @@ class Connection(object): # TODO support auth for pooling
 
     def __init__(self, host=None, port=None, pool_size=None,
                  auto_start_request=None, timeout=None, slave_okay=False,
-                 network_timeout=None, _connect=True):
+                 network_timeout=None, document_class=dict, _connect=True):
         """Create a new connection to a single MongoDB instance at *host:port*.
 
         The resultant connection object has connection-pooling built in. It
@@ -130,13 +126,17 @@ class Connection(object): # TODO support auth for pooling
           - `port` (optional): port number on which to connect
           - `pool_size` (optional): DEPRECATED
           - `auto_start_request` (optional): DEPRECATED
-          - `slave_okay` (optional): is it okay to connect directly to and
-            perform queries on a slave instance
+          - `slave_okay` (optional): is it okay to connect directly to
+            and perform queries on a slave instance
           - `timeout` (optional): DEPRECATED
-          - `network_timeout` (optional): timeout (in seconds) to use for socket
-            operations - default is no timeout
+          - `network_timeout` (optional): timeout (in seconds) to use
+            for socket operations - default is no timeout
+          - `document_class` (optional): default class to use for
+            documents returned from queries on this connection
 
         .. seealso:: :meth:`end_request`
+        .. versionadded:: 1.7
+           The `document_class` parameter.
         .. versionchanged:: 1.4
            DEPRECATED The `pool_size`, `auto_start_request`, and `timeout`
            parameters.
@@ -176,6 +176,7 @@ class Connection(object): # TODO support auth for pooling
         self.__pool = Pool(self.__connect)
 
         self.__network_timeout = network_timeout
+        self.__document_class = document_class
 
         # cache of existing indexes used by ensure_index ops
         self.__index_cache = {}
@@ -188,7 +189,8 @@ class Connection(object): # TODO support auth for pooling
         i = source.find(sub)
         if i == -1:
             return (source, None)
-        return (source[:i], source[i+len(sub):])
+
+        return (source[:i], source[i + len(sub):])
 
     @staticmethod
     def _parse_uri(uri):
@@ -197,7 +199,8 @@ class Connection(object): # TODO support auth for pooling
         if uri.startswith("mongodb://"):
             uri = uri[len("mongodb://"):]
         elif "://" in uri:
-            raise InvalidURI("Invalid uri scheme: %s" % Connection.__partition(uri, "://")[0])
+            raise InvalidURI("Invalid uri scheme: %s"
+                             % Connection.__partition(uri, "://")[0])
 
         (hosts, database) = Connection.__partition(uri, "/")
 
@@ -210,7 +213,8 @@ class Connection(object): # TODO support auth for pooling
             (auth, hosts) = Connection.__partition(hosts, "@")
 
             if ":" not in auth:
-                raise InvalidURI("auth must be specified as 'username:password@'")
+                raise InvalidURI("auth must be specified as "
+                                 "'username:password@'")
             (username, password) = Connection.__partition(auth, ":")
 
         host_list = []
@@ -311,6 +315,9 @@ class Connection(object): # TODO support auth for pooling
                 warnings.warn("The %s parameter to Connection.paired is "
                               "deprecated" % param, DeprecationWarning)
 
+        if "slave_okay" in connection_args:
+            raise TypeError("cannot specify slave_okay on paired connections")
+
         connection_args['_connect'] = False
 
         connection = cls(left[0], left[1], **connection_args)
@@ -318,24 +325,9 @@ class Connection(object): # TODO support auth for pooling
         return connection
 
     def __master(self, sock):
-        """Get the hostname and port of the master Mongo instance.
-
-        Return a tuple (host, port).
+        """Is this socket connected to a master server?
         """
-        result = self["admin"].command("ismaster", _sock=sock)
-
-        if result["ismaster"] == 1:
-            return True
-        else:
-            if "remote" not in result:
-                return False
-
-            strings = result["remote"].split(":", 1)
-            if len(strings) == 1:
-                port = self.PORT
-            else:
-                port = int(strings[1])
-            return (strings[0], port)
+        return self["admin"].command("ismaster", _sock=sock)["ismaster"]
 
     def _cache_index(self, database, collection, index, ttl):
         """Add an index to the index cache for ensure_index operations.
@@ -396,20 +388,18 @@ class Connection(object): # TODO support auth for pooling
         """Current connected host.
 
         .. versionchanged:: 1.3
-           ``host`` is now a property rather than a method. The ``host()``
-           method is deprecated.
+           ``host`` is now a property rather than a method.
         """
-        return helpers.callable_value(self.__host, "Connection.host")
+        return self.__host
 
     @property
     def port(self):
         """Current connected port.
 
         .. versionchanged:: 1.3
-           ``port`` is now a property rather than a method. The ``port()``
-           method is deprecated.
+           ``port`` is now a property rather than a method.
         """
-        return helpers.callable_value(self.__port, "Connection.port")
+        return self.__port
 
     @property
     def slave_okay(self):
@@ -417,6 +407,19 @@ class Connection(object): # TODO support auth for pooling
         """
         return self.__slave_okay
 
+    def get_document_class(self):
+        return self.__document_class
+
+    def set_document_class(self, klass):
+        self.__document_class = klass
+
+    document_class = property(get_document_class, set_document_class,
+                              doc="""Default class to use for documents
+                              returned from queries on this connection.
+
+                              .. versionadded:: 1.7
+                              """)
+
     def __find_master(self):
         """Create a new socket and use it to figure out who the master is.
 
@@ -426,6 +429,7 @@ class Connection(object): # TODO support auth for pooling
         self.__host = None
         self.__port = None
         sock = None
+        sock_error = False
         for (host, port) in self.__nodes:
             try:
                 try:
@@ -435,25 +439,20 @@ class Connection(object): # TODO support auth for pooling
                     sock.connect((host, port))
                     sock.settimeout(self.__network_timeout)
                     master = self.__master(sock)
-                    if master is True or self.__slave_okay:
+                    if master or self.__slave_okay:
                         self.__host = host
                         self.__port = port
                         return
-                    if not master:
-                        raise ConfigurationError("trying to connect directly to"
-                                                 " slave %s:%r - must specify "
-                                                 "slave_okay to connect to "
-                                                 "slaves" % (host, port))
-                    if master not in self.__nodes:
-                        raise ConfigurationError("%r claims master is %r, "
-                                                 "but that's not configured" %
-                                                 ((host, port), master))
                 except socket.error, e:
-                    continue
+                    sock_error = True
             finally:
                 if sock is not None:
                     sock.close()
-        raise AutoReconnect("could not find master")
+        if sock_error or self.__host is None:
+            raise AutoReconnect("could not find master")
+        raise ConfigurationError("No master node in %r. You must specify "
+                                 "slave_okay to connect to "
+                                 "slaves." % self.__nodes)
 
     def __connect(self):
         """(Re-)connect to Mongo and return a new (connected) socket.
@@ -503,9 +502,10 @@ class Connection(object): # TODO support auth for pooling
         """Set this connection's cursor manager.
 
         Raises :class:`TypeError` if `manager_class` is not a subclass of
-        :class:`~pymongo.cursor_manager.CursorManager`. A cursor manager handles
-        closing cursors. Different managers can implement different policies in
-        terms of when to actually kill a cursor that has been closed.
+        :class:`~pymongo.cursor_manager.CursorManager`. A cursor manager
+        handles closing cursors. Different managers can implement different
+        policies in terms of when to actually kill a cursor that has
+        been closed.
 
         :Parameters:
           - `manager_class`: cursor manager to use
@@ -566,7 +566,8 @@ class Connection(object): # TODO support auth for pooling
             # lastError) and raise OperationFailure if it is an error
             # response.
             if with_last_error:
-                response = self.__receive_message_on_socket(1, request_id, sock)
+                response = self.__receive_message_on_socket(1, request_id,
+                                                            sock)
                 return self.__check_response_to_last_error(response)
             return None
         except (ConnectionFailure, socket.error), e:
@@ -731,13 +732,6 @@ class Connection(object): # TODO support auth for pooling
             raise TypeError("cursor_ids must be a list")
         return self._send_message(message.kill_cursors(cursor_ids))
 
-    def __database_info(self):
-        """Get a dictionary of (database_name: size_on_disk).
-        """
-        result = self["admin"].command("listDatabases")
-        info = result["databases"]
-        return dict([(db["name"], db["sizeOnDisk"]) for db in info])
-
     def server_info(self):
         """Get information about the MongoDB server we're connected to.
         """
@@ -746,7 +740,8 @@ class Connection(object): # TODO support auth for pooling
     def database_names(self):
         """Get a list of the names of all databases on the connected server.
         """
-        return self.__database_info().keys()
+        return [db["name"] for db in
+                self.admin.command("listDatabases")["databases"]]
 
     def drop_database(self, name_or_database):
         """Drop a database.
