@@ -12,17 +12,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""Little bits and pieces used by the driver that don't really fit elsewhere."""
+"""Bits and pieces used by the driver that don't really fit elsewhere."""
 
 try:
     import hashlib
     _md5func = hashlib.md5
-except: # for Python < 2.5
+except:  # for Python < 2.5
     import md5
     _md5func = md5.new
 import struct
-import sys
-import warnings
 
 import pymongo
 from pymongo import bson
@@ -30,6 +28,7 @@ from pymongo.errors import (OperationFailure,
                             AutoReconnect)
 from pymongo.son import SON
 
+
 def _index_list(key_or_list, direction=None):
     """Helper to generate a list of (key, direction) pairs.
 
@@ -72,7 +71,7 @@ def _index_document(index_list):
     return index
 
 
-def _unpack_response(response, cursor_id=None):
+def _unpack_response(response, cursor_id=None, as_class=dict):
     """Unpack a response from the database.
 
     Check the response for errors and unpack, returning a dictionary
@@ -83,6 +82,7 @@ def _unpack_response(response, cursor_id=None):
       - `cursor_id` (optional): cursor_id we sent to get this response -
         used for raising an informative exception when we get cursor id not
         valid at server response
+      - `as_class` (optional): class to use for resulting documents
     """
     response_flag = struct.unpack("<i", response[:4])[0]
     if response_flag & 1:
@@ -102,7 +102,7 @@ def _unpack_response(response, cursor_id=None):
     result["cursor_id"] = struct.unpack("<q", response[4:12])[0]
     result["starting_from"] = struct.unpack("<i", response[12:16])[0]
     result["number_returned"] = struct.unpack("<i", response[16:20])[0]
-    result["data"] = bson._to_dicts(response[20:])
+    result["data"] = bson._to_dicts(response[20:], as_class)
     assert len(result["data"]) == result["number_returned"]
     return result
 
@@ -130,36 +130,19 @@ def _auth_key(nonce, username, password):
     return unicode(md5hash.hexdigest())
 
 
-# These two functions are some magic to get values we can use for deprecating
-# method style access in favor of property style access while remaining
-# backwards compatible.
-def __prop_call(self, *args, **kwargs):
-    warnings.warn("'%s()' has been deprecated and will be removed. "
-                  "Please use '%s' instead." %
-                  (self.__prop_name, self.__prop_name),
-                  DeprecationWarning)
-    return self
+def _fields_list_to_dict(fields):
+    """Takes a list of field names and returns a matching dictionary.
 
-__class_cache = {}
+    ["a", "b"] becomes {"a": 1, "b": 1}
 
-def callable_value(value, prop_name):
-    t = type(value)
+    and
 
-    if "CallableVal" in str(t):
-        return value
-
-    if (t, prop_name) in __class_cache:
-        cls = __class_cache[(t, prop_name)]
-    else:
-        cls = type.__new__(type, "CallableVal", (t,),
-                           {"__call__": __prop_call,
-                            "__prop_name": prop_name})
-        __class_cache[(t, prop_name)] = cls
-
-    try:
-        # This works for regular classes
-        value.__class__ = cls
-        return value
-    except:
-        # This works for builtins
-        return cls(value)
+    ["a.b.c", "d", "a.c"] becomes {"a.b.c": 1, "d": 1, "a.c": 1}
+    """
+    as_dict = {}
+    for field in fields:
+        if not isinstance(field, basestring):
+            raise TypeError("fields must be a list of key names as "
+                            "(string, unicode)")
+        as_dict[field] = 1
+    return as_dict
