@@ -24,8 +24,6 @@ MongoDB.
 
 import random
 import struct
-import sys
-import threading
 
 from pymongo import bson
 try:
@@ -50,10 +48,10 @@ def __pack_message(operation, data):
 
     Returns the resultant message string.
     """
-    request_id = random.randint(-2**31 - 1, 2**31)
+    request_id = random.randint(-2 ** 31 - 1, 2 ** 31)
     message = struct.pack("<i", 16 + len(data))
     message += struct.pack("<i", request_id)
-    message += __ZERO # responseTo
+    message += __ZERO  # responseTo
     message += struct.pack("<i", operation)
     return (request_id, message + data)
 
