@@ -29,7 +29,7 @@ sys.path[0:0] = [""]
 
 from nose.plugins.skip import SkipTest
 
-import qcheck
+import pymongo
 from pymongo.binary import Binary
 from pymongo.code import Code
 from pymongo.objectid import ObjectId
@@ -38,7 +38,16 @@ from pymongo.son import SON
 from pymongo.timestamp import Timestamp
 from pymongo.bson import BSON, is_valid, _to_dicts
 from pymongo.errors import InvalidDocument, InvalidStringData
+from pymongo.max_key import MaxKey
+from pymongo.min_key import MinKey
+import qcheck
 
+class SomeZone(datetime.tzinfo):
+    def utcoffset(self, dt):
+        return datetime.timedelta(minutes=555)
+    def dst(self, dt):
+        # a fixed-offset class:  doesn't account for DST
+        return datetime.timedelta(0)
 
 class TestBSON(unittest.TestCase):
 
@@ -108,6 +117,9 @@ class TestBSON(unittest.TestCase):
         self.assertEqual(BSON.from_dict({"none": {}}),
                          "\x10\x00\x00\x00\x03\x6E\x6F\x6E\x65\x00\x05\x00\x00"
                          "\x00\x00\x00")
+        self.assertEqual(BSON.from_dict({"test": Binary("test", 0)}),
+                         "\x14\x00\x00\x00\x05\x74\x65\x73\x74\x00\x04\x00\x00"
+                         "\x00\x00\x74\x65\x73\x74\x00")
         self.assertEqual(BSON.from_dict({"test": Binary("test")}),
                          "\x18\x00\x00\x00\x05\x74\x65\x73\x74\x00\x08\x00\x00"
                          "\x00\x02\x04\x00\x00\x00\x74\x65\x73\x74\x00")
@@ -164,6 +176,8 @@ class TestBSON(unittest.TestCase):
         helper({"ref": DBRef("coll", 5)})
         helper({"ref": DBRef("coll", 5, "foo")})
         helper({"ref": Timestamp(1,2)})
+        helper({"foo": MinKey()})
+        helper({"foo": MaxKey()})
 
         def from_then_to_dict(dict):
             return dict == (BSON.from_dict(dict)).to_dict()
@@ -171,6 +185,14 @@ class TestBSON(unittest.TestCase):
         qcheck.check_unittest(self, from_then_to_dict,
                               qcheck.gen_mongo_dict(3))
 
+    def test_aware_datetime(self):
+        aware = datetime.datetime(1993, 4, 4, 2, tzinfo=SomeZone())
+        utc_naive = (aware - aware.utcoffset()).replace(tzinfo=None)
+        self.assertEqual(datetime.datetime(1993, 4, 3, 16, 45), utc_naive)
+        after = BSON.from_dict({"date": aware}).to_dict()["date"]
+        self.assertEqual(None, after.tzinfo)
+        self.assertEqual(utc_naive, after)
+
     def test_bad_encode(self):
         self.assertRaises(InvalidStringData, BSON.from_dict,
                           {"lalala": '\xf4\xe0\xf0\xe1\xc0 Color Touch'})
@@ -248,15 +270,24 @@ class TestBSON(unittest.TestCase):
                          BSON.from_dict(SON([("b", SON([("a", "a"), ("_id", "a")])),
                                              ("_id", "b")])))
 
+    def test_dates(self):
+        doc = {"early": datetime.datetime(1686, 5, 5),
+               "late": datetime.datetime(2086, 5, 5)}
+        try:
+            self.assertEqual(doc, BSON.from_dict(doc).to_dict())
+        except ValueError:
+            # Ignore ValueError when no C ext, since it's probably
+            # a problem w/ 32-bit Python - we work around this in the
+            # C ext, though.
+            if pymongo.has_c():
+                raise
 
-# TODO this test doesn't pass w/ C extension
-#
-# timegm doesn't handle years < 1900 (negative), at least on OS X
-# we probably need to use our own version of timegm
+    def test_custom_class(self):
+        self.assert_(isinstance(BSON.from_dict({}).to_dict(), dict))
+        self.failIf(isinstance(BSON.from_dict({}).to_dict(), SON))
+        self.assert_(isinstance(BSON.from_dict({}).to_dict(SON), SON))
 
-#     def test_date_before_epoch(self):
-#         doc = {"date": datetime.datetime(1600, 5, 5)}
-#         self.assertEqual(doc, BSON.from_dict(doc).to_dict())
+        self.assertEqual(1, BSON.from_dict({"x": 1}).to_dict(SON)["x"])
 
 
 if __name__ == "__main__":
