@@ -227,30 +227,24 @@ class TestCollection(unittest.TestCase):
     def test_index_info(self):
         db = self.db
         db.test.drop_indexes()
+        db.test.remove({})
+        db.test.save({}) # create collection
         self.assertEqual(len(db.test.index_information()), 1)
         self.assert_("_id_" in db.test.index_information())
 
         db.test.create_index("hello")
         self.assertEqual(len(db.test.index_information()), 2)
         self.assertEqual(db.test.index_information()["hello_1"],
-                         [("hello", ASCENDING)])
+                         {"key": [("hello", ASCENDING)]})
 
-        db.test.create_index([("hello", DESCENDING), ("world", ASCENDING)])
+        db.test.create_index([("hello", DESCENDING), ("world", ASCENDING)],
+                             unique=True)
         self.assertEqual(db.test.index_information()["hello_1"],
-                         [("hello", ASCENDING)])
+                         {"key": [("hello", ASCENDING)]})
         self.assertEqual(len(db.test.index_information()), 3)
-        self.assert_(("hello", DESCENDING) in
-                     db.test.index_information()["hello_-1_world_1"])
-        self.assert_(("world", ASCENDING) in
-                     db.test.index_information()["hello_-1_world_1"])
-        self.assert_(len(db.test.index_information()["hello_-1_world_1"]) == 2)
-
-    def test_fields_list_to_dict(self):
-        f = self.db.test._fields_list_to_dict
-
-        self.assertEqual(f(["a", "b"]), {"a": 1, "b": 1})
-        self.assertEqual(f(["a.b.c", "d", "a.c"]),
-                         {"a.b.c": 1, "d": 1, "a.c": 1})
+        self.assertEqual({"key": [("hello", DESCENDING), ("world", ASCENDING)],
+                          "unique": True},
+                         db.test.index_information()["hello_-1_world_1"])
 
     def test_field_selection(self):
         db = self.db
@@ -343,6 +337,20 @@ class TestCollection(unittest.TestCase):
         self.assert_("mike" in db.test.find({}, ["mike"]).next())
         self.failIf("extra thing" in db.test.find({}, ["mike"]).next())
 
+    def test_fields_specifier_as_dict(self):
+        db = self.db
+        db.test.remove({})
+
+        db.test.insert({"x": [1, 2, 3], "mike": "awesome"})
+
+        self.assertEqual([1,2,3], db.test.find_one()["x"])
+        if version.at_least(db.connection, (1, 5, 1)):
+            self.assertEqual([2,3],
+                             db.test.find_one(fields={"x": {"$slice":
+                                                            -2}})["x"])
+        self.assert_("x" not in db.test.find_one(fields={"x": 0}))
+        self.assert_("mike" in db.test.find_one(fields={"x": 0}))
+
     def test_find_w_regex(self):
         db = self.db
         db.test.remove({})
@@ -807,6 +815,10 @@ class TestCollection(unittest.TestCase):
             self.assertEqual(x, doc["x"])
             x += 1
 
+        db.test.insert({})
+        self.assertRaises(OperationFailure, db.foo.rename, "test")
+        db.foo.rename("test", dropTarget=True)
+
     # doesn't really test functionality, just that the option is set correctly
     def test_snapshot(self):
         db = self.db
@@ -835,7 +847,58 @@ class TestCollection(unittest.TestCase):
         self.assertEqual(None, db.test.find_one({"hello": "foo"}))
         self.assertEqual(None, db.test.find_one(ObjectId()))
 
-        self.assertRaises(TypeError, db.test.find_one, 6)
+    def test_find_one_non_objectid(self):
+        db = self.db
+        db.drop_collection("test")
+
+        db.test.save({"_id": 5})
+
+        self.assert_(db.test.find_one(5))
+        self.failIf(db.test.find_one(6))
+
+    def test_remove_non_objectid(self):
+        db = self.db
+        db.drop_collection("test")
+
+        db.test.save({"_id": 5})
+
+        self.assertEqual(1, db.test.count())
+        db.test.remove(5)
+        self.assertEqual(0, db.test.count())
+
+    def test_find_one_with_find_args(self):
+        db = self.db
+        db.drop_collection("test")
+
+        db.test.save({"x": 1})
+        db.test.save({"x": 2})
+        db.test.save({"x": 3})
+
+        self.assertEqual(1, db.test.find_one()["x"])
+        self.assertEqual(2, db.test.find_one(skip=1, limit=2)["x"])
+
+    def test_find_with_sort(self):
+        db = self.db
+        db.drop_collection("test")
+
+        db.test.save({"x": 2})
+        db.test.save({"x": 1})
+        db.test.save({"x": 3})
+
+        self.assertEqual(2, db.test.find_one()["x"])
+        self.assertEqual(1, db.test.find_one(sort=[("x", 1)])["x"])
+        self.assertEqual(3, db.test.find_one(sort=[("x", -1)])["x"])
+
+        def to_list(foo):
+            return [bar["x"] for bar in foo]
+
+        self.assertEqual([2,1,3], to_list(db.test.find()))
+        self.assertEqual([1,2,3], to_list(db.test.find(sort=[("x", 1)])))
+        self.assertEqual([3,2,1], to_list(db.test.find(sort=[("x", -1)])))
+
+        self.assertRaises(TypeError, db.test.find, sort=5)
+        self.assertRaises(TypeError, db.test.find, sort="hello")
+        self.assertRaises(ValueError, db.test.find, sort=["hello", 1])
 
     def test_insert_adds_id(self):
         doc = {"hello": "world"}
@@ -963,6 +1026,30 @@ class TestCollection(unittest.TestCase):
         self.db.drop_collection("test")
         self.db.test.drop_indexes()
 
+    # This is really a bson test but easier to just reproduce it here...
+    # (Shame on me)
+    def test_bad_encode(self):
+        c = self.db.test
+        warnings.simplefilter("ignore")
+        self.assertRaises(InvalidDocument, c.save, {"x": c})
+        warnings.simplefilter("default")
+
+    def test_as_class(self):
+        c = self.db.test
+        c.remove()
+        c.insert({"x": 1})
+
+        self.assert_(isinstance(c.find().next(), dict))
+        self.failIf(isinstance(c.find().next(), SON))
+        self.assert_(isinstance(c.find(as_class=SON).next(), SON))
+
+        self.assert_(isinstance(c.find_one(), dict))
+        self.failIf(isinstance(c.find_one(), SON))
+        self.assert_(isinstance(c.find_one(as_class=SON), SON))
+
+        self.assertEqual(1, c.find_one(as_class=SON)["x"])
+        self.assertEqual(1, c.find(as_class=SON).next()["x"])
+
 
 if __name__ == "__main__":
     unittest.main()
