@@ -30,6 +30,7 @@ from pymongo.errors import (AutoReconnect,
                             InvalidName,
                             InvalidURI,
                             OperationFailure)
+from pymongo.son import SON
 from test import version
 
 
@@ -212,45 +213,6 @@ class TestConnection(unittest.TestCase):
             except AssertionError:
                 self.fail()
 
-    # NOTE this probably doesn't all belong in this file, but it's easier in
-    # one place
-    def test_deprecated_method_for_attr(self):
-        c = Connection(self.host, self.port)
-        db = c.foo
-        coll = db.bar
-
-        warnings.simplefilter("error")
-
-        self.assertRaises(DeprecationWarning, c.host)
-        self.assertRaises(DeprecationWarning, c.port)
-        self.assertRaises(DeprecationWarning, db.connection)
-        self.assertRaises(DeprecationWarning, db.name)
-        self.assertRaises(DeprecationWarning, coll.full_name)
-        self.assertRaises(DeprecationWarning, coll.name)
-        self.assertRaises(DeprecationWarning, coll.database)
-
-        warnings.resetwarnings()
-        warnings.simplefilter("ignore")
-
-        self.assertEqual(c.host, c.host())
-        self.assertEqual(c.port, c.port())
-        self.assertEqual(db.connection, db.connection())
-        self.assertEqual(db.name, db.name())
-        self.assertEqual(coll.full_name, coll.full_name())
-        self.assertEqual(coll.name, coll.name())
-        self.assertEqual(coll.database, coll.database())
-
-        warnings.resetwarnings()
-        warnings.simplefilter("default")
-
-        self.assertEqual(self.host, c.host)
-        self.assertEqual(self.port, c.port)
-        self.assertEqual(c, db.connection)
-        self.assertEqual("foo", db.name)
-        self.assertEqual("foo.bar", coll.full_name)
-        self.assertEqual("bar", coll.name)
-        self.assertEqual(db, coll.database)
-
     def test_disconnect(self):
         c = Connection(self.host, self.port)
         coll = c.foo.bar
@@ -386,6 +348,34 @@ class TestConnection(unittest.TestCase):
         except EOFError:
             pass
 
+    def test_document_class(self):
+        c = Connection(self.host, self.port)
+        db = c.pymongo_test
+        db.test.insert({"x": 1})
+
+        self.assertEqual(dict, c.document_class)
+        self.assert_(isinstance(db.test.find_one(), dict))
+        self.failIf(isinstance(db.test.find_one(), SON))
+
+        c.document_class = SON
+
+        self.assertEqual(SON, c.document_class)
+        self.assert_(isinstance(db.test.find_one(), SON))
+        self.failIf(isinstance(db.test.find_one(as_class=dict), SON))
+
+        c = Connection(self.host, self.port, document_class=SON)
+        db = c.pymongo_test
+
+        self.assertEqual(SON, c.document_class)
+        self.assert_(isinstance(db.test.find_one(), SON))
+        self.failIf(isinstance(db.test.find_one(as_class=dict), SON))
+
+        c.document_class = dict
+
+        self.assertEqual(dict, c.document_class)
+        self.assert_(isinstance(db.test.find_one(), dict))
+        self.failIf(isinstance(db.test.find_one(), SON))
+
 # TODO come up with a different way to test `network_timeout`. This is just
 # too sketchy.
 #
