@@ -197,7 +197,10 @@ class TestDatabase(unittest.TestCase):
         prev_error = db.previous_error()
         self.assertEqual(prev_error["nPrev"], 1)
         del prev_error["nPrev"]
-        self.assertEqual(db.error(), prev_error)
+        prev_error.pop("lastOp", None)
+        error = db.error()
+        error.pop("lastOp", None)
+        self.assertEqual(error, prev_error)
 
         db.test.find_one()
         self.assertEqual(None, db.error())
@@ -364,10 +367,6 @@ class TestDatabase(unittest.TestCase):
         db = self.connection.pymongo_test
         db.test.remove({})
 
-        self.assertRaises(TypeError, db.test.remove, 5)
-        self.assertRaises(TypeError, db.test.remove, "test")
-        self.assertRaises(TypeError, db.test.remove, [])
-
         one = db.test.save({"x": 1})
         db.test.save({"x": 2})
         db.test.save({"x": 3})
