@@ -1,4 +1,4 @@
-# Copyright 2009 10gen, Inc.
+# Copyright 2009-2010 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,7 +20,7 @@ Only really intended to be used by internal build scripts.
 import sys
 sys.path[0:0] = [""]
 
-try:
-    from pymongo import _cbson
-except ImportError:
+import pymongo
+
+if not pymongo.has_c():
     sys.exit("could not import _cbson")
